"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDependencyAnalyzer = void 0;
const ts_morph_1 = require("ts-morph");
const is_builtin_module_1 = __importDefault(require("is-builtin-module"));
const micromatch_1 = __importDefault(require("micromatch"));
const ts_morph_helpers_1 = require("ts-morph-helpers");
const compact = (collection) => Array.from(new Set(collection)).filter((value) => Boolean(value));
const findRequireModuleSpecifiers = (sourceFile) => (0, ts_morph_helpers_1.findCallExpressionsByName)(sourceFile, 'require').map(expression => expression.getFirstDescendantByKind(ts_morph_1.ts.SyntaxKind.StringLiteral));
const isExternalDependency = (moduleSpecifier, tsConfigPathGlobs) => {
    if (moduleSpecifier.startsWith('.'))
        return false;
    if ((0, is_builtin_module_1.default)(moduleSpecifier))
        return false;
    if (tsConfigPathGlobs.length > 0 && micromatch_1.default.isMatch(moduleSpecifier, tsConfigPathGlobs))
        return false;
    return true;
};
const resolvePackageName = (moduleSpecifier) => {
    const parts = moduleSpecifier.split('/').slice(0, 2);
    return moduleSpecifier.startsWith('@') ? parts.join('/') : parts[0];
};
const getDependencyAnalyzer = (configuration) => {
    const { dependencies, devDependencies, peerDependencies, optionalDependencies, tsConfigPathGlobs } = configuration;
    const productionDependencies = [...dependencies, ...peerDependencies, ...optionalDependencies];
    const referencedDependencies = new Set();
    const getUnresolvedDependencies = (sourceFile) => {
        const unresolvedDependencies = new Set();
        const importLiterals = sourceFile.getImportStringLiterals();
        const requireCallExpressions = findRequireModuleSpecifiers(sourceFile);
        const moduleSpecifiers = compact([importLiterals, requireCallExpressions].flat()).map(i => i.getLiteralText());
        moduleSpecifiers.forEach(moduleSpecifier => {
            if (!isExternalDependency(moduleSpecifier, tsConfigPathGlobs))
                return;
            const packageName = resolvePackageName(moduleSpecifier);
            if (!productionDependencies.includes(packageName) && !devDependencies.includes(packageName)) {
                unresolvedDependencies.add({ filePath: sourceFile.getFilePath(), symbol: moduleSpecifier });
            }
            if (productionDependencies.includes(packageName) || devDependencies.includes(packageName)) {
                referencedDependencies.add(packageName);
            }
        });
        return unresolvedDependencies;
    };
    const getUnusedDependencies = () => productionDependencies.filter(dependency => !referencedDependencies.has(dependency));
    const getUnusedDevDependencies = () => devDependencies.filter(dependency => !referencedDependencies.has(dependency));
    return { getUnresolvedDependencies, getUnusedDependencies, getUnusedDevDependencies };
};
exports.getDependencyAnalyzer = getDependencyAnalyzer;
